@@ -16,7 +16,7 @@ from socket import (
     getaddrinfo,
     socket,
 )
-from threading import Timer, Thread, Lock
+from threading import Timer, Thread, Event
 from time import sleep, time
 from wsgiref import simple_server
 
@@ -144,26 +144,24 @@ class Server:
             if not self.running:
                 break
 
-            lock = Lock()
-            lock.acquire()
-
-            def request_received(environ):
-                lock.release()
-                if hasattr(callbacks, "request_received"):
-                    callbacks.request_received(environ)
-
-            def runner():
-                try:
-                    self._fetch_and_run_one_job(request_received)
-                finally:
-                    if lock.locked():
-                        lock.release()
-                    if hasattr(callbacks, "done"):
-                        callbacks.done()
-
-            t = Thread(target=runner)
+            event = Event()
+            t = Thread(target=self._run_one_poll, args=(event, callbacks))
             t.start()
-            lock.acquire()
+            event.wait()
+
+    def _run_one_poll(self, event, callbacks):
+        def request_received(environ):
+            event.set()
+            next = getattr(callbacks, 'request_received', None)
+            if next:
+                next(environ)
+
+        try:
+            self._fetch_and_run_one_job(request_received)
+        finally:
+            event.set()
+            if hasattr(callbacks, "done"):
+                callbacks.done()
 
 
 def install_term_handler(f):
